@@ -2,8 +2,8 @@
 # 主要模型
 
 import numpy as np
-from bert4keras.layers import *
-from bert4keras.snippets import delete_arguments
+from my_bert.layers import *
+from my_bert.snippets import delete_arguments
 from keras.models import Model
 import json
 
@@ -218,7 +218,7 @@ class Transformer(object):
                 head_size = self.attention_head_size
                 key_size = self.attention_key_size
                 W = np.linalg.qr(np.random.randn(key_size, head_size))[0].T
-                if layer.scaled_dot_product:
+                if layer.attention_scale:
                     W = W * key_size**0.25 / head_size**0.25
                 for i in range(count):
                     w, v = weights[i], values[i]
@@ -357,7 +357,7 @@ class BERT(Transformer):
 
         attention_name = 'Transformer-%d-MultiHeadSelfAttention' % index
         feed_forward_name = 'Transformer-%d-FeedForward' % index
-        attention_mask = self.compute_attention_mask()
+        attention_mask = self.compute_attention_mask(index)
 
         # Self Attention
         xi, x, arguments = x, [x, x, x], {'a_mask': None}
@@ -598,6 +598,7 @@ class BERT(Transformer):
 
 
 
+
 def extend_with_language_model(BaseModel):
     """添加下三角的Attention Mask（语言模型用）
     """
@@ -614,14 +615,11 @@ def extend_with_language_model(BaseModel):
             if self.attention_mask is None:
 
                 def lm_mask(s):
-                    import tensorflow as tf
                     seq_len = K.shape(s)[1]
-                    with K.name_scope('attention_mask'):
-                        # 用K.ones可能会有问题
-                        # 参考 https://github.com/tensorflow/tensorflow/issues/24938
-                        ones = tf.ones((1, 1, seq_len, seq_len))
-                    a_mask = tf.linalg.band_part(ones, -1, 0)
-                    return a_mask
+                    idxs = K.arange(0, seq_len)
+                    mask = idxs[None, :] <= idxs[:, None]
+                    mask = K.cast(mask, K.floatx())
+                    return mask[None, None]
 
                 self.attention_mask = self.apply(
                     inputs=self.inputs[1],
@@ -635,6 +633,88 @@ def extend_with_language_model(BaseModel):
     return LanguageModel
 
 
+def extend_with_unified_language_model(BaseModel):
+    """添加UniLM的Attention Mask（UnifiedLanguageModel用）
+    """
+    class UnifiedLanguageModel(BaseModel):
+        """带UniLM的Attention Mask的派生模型
+        UniLM: https://arxiv.org/abs/1905.03197
+        """
+        def __init__(self, *args, **kwargs):
+            super(UnifiedLanguageModel, self).__init__(*args, **kwargs)
+            self.with_mlm = self.with_mlm or True
+
+        def compute_attention_mask(self, inputs=None):
+            """重载此函数即可
+            """
+            if self.attention_mask is None:
+
+                def unilm_mask(s):
+                    idxs = K.cumsum(s, axis=1)
+                    mask = idxs[:, None, :] <= idxs[:, :, None]
+                    mask = K.cast(mask, K.floatx())
+                    return mask[:, None]
+
+                self.attention_mask = self.apply(
+                    inputs=self.inputs[1],
+                    layer=Lambda,
+                    function=unilm_mask,
+                    name='Attention-UniLM-Mask'
+                )
+
+            return self.attention_mask
+
+    return UnifiedLanguageModel
+
+
+def extend_with_parallel_unified_language_model(BaseModel):
+    """添加并行式UniLM的Attention Mask（Seq2Seq用）
+    注：用于加速一对多的Seq2Seq任务的训练过程。
+    """
+    class ParallelUnifiedLanguageModel(BaseModel):
+        """带并行式UniLM的Attention Mask的派生模型
+        UniLM: https://arxiv.org/abs/1905.03197
+        """
+        def __init__(self, *args, **kwargs):
+            super(ParallelUnifiedLanguageModel, self).__init__(*args, **kwargs)
+            self.with_mlm = self.with_mlm or True
+            self.custom_position_ids = True
+
+        def compute_attention_mask(self, inputs=None):
+            """重载此函数即可
+            """
+            if self.attention_mask is None:
+
+                def punilm_mask(inputs):
+                    s, p = inputs
+
+                    s_idxs = K.cumsum(s, axis=1)
+                    s_mask = s_idxs[:, None, :] <= s_idxs[:, :, None]
+                    s_mask = K.cast(s_mask, K.floatx())
+
+                    p = K.less(p[:, 1:] - p[:, :-1], 0)
+                    p = K.cast(p, K.floatx())
+                    p = K.concatenate([K.zeros_like(p[:, :1]), p], axis=1)
+                    p_idxs = K.cumsum(p, axis=1) + s
+                    p_mask = K.equal(p_idxs[:, None, :], p_idxs[:, :, None])
+                    p_mask = K.cast(p_mask, K.floatx())
+
+                    s = s[:, None]
+                    mask = s_mask * (1 - s) + p_mask * s_mask * s
+                    return mask[:, None]
+
+                self.attention_mask = self.apply(
+                    inputs=self.inputs[1:3],
+                    layer=Lambda,
+                    function=punilm_mask,
+                    name='Attention-PUniLM-Mask'
+                )
+
+            return self.attention_mask
+
+    return ParallelUnifiedLanguageModel
+
+
 def build_transformer_model(
     config_path=None,
     checkpoint_path=None,
@@ -657,13 +737,17 @@ def build_transformer_model(
     model, application = model.lower(), application.lower()
 
     models = {
-        'bert': BERT,
+        'bert': BERT
     }
     MODEL = models[model]
 
     if model != 't5':
         if application == 'lm':
             MODEL = extend_with_language_model(MODEL)
+        elif application == 'unilm':
+            MODEL = extend_with_unified_language_model(MODEL)
+        elif application == 'punilm':
+            MODEL = extend_with_parallel_unified_language_model(MODEL)
 
     transformer = MODEL(**configs)
     transformer.build(**configs)
